# Proxying a SOAP API #

This example shows how to proxy your API. Applications send service requests to your proxy, which in turn calls the real API.

### Assumptions ###

This document assumes that you are familiar with Mule and the [Anypoint™ Studio interface](http://www.mulesoft.org/documentation/display/current/Anypoint+Studio+Essentials). To increase your familiarity with Studio, consider completing one or more [Anypoint Studio Tutorials](http://www.mulesoft.org/documentation/display/current/Basic+Studio+Tutorial). Further, this example assumes that you have a basic understanding of [Mule flows](http://www.mulesoft.org/documentation/display/current/Mule+Application+Architecture) and [Mule Global Elements](http://www.mulesoft.org/documentation/display/current/Global+Elements).

Furthermore, this document assumes that you have a SOAP API that has not been built to run on Mule ESB

This document describes the details of the example within the context of Anypoint Studio, Mule ESB’s graphical user interface, and includes configuration details for XML Editor where relevant.

### Example Use Case ###

In this simple example, an application serves as a minimum proxy that can route requests and responses to and from a SOAP API. After receiving requests, it creates a shopping cart and returns its ID.

### Set Up and Run the Example ###

Complete the following procedure to create, then run this example in your own instance of Anypoint Studio. Skip ahead to the next section if you prefer to simply examine this example via code snippets.

1. [Create](http://www.mulesoft.org/documentation/display/current/Mule+Examples#MuleExamples-CreateandRunExampleApplications) the example application in Anypoint Studio and run it.
1. Send the following request to your API via the proxy URL: http://localhost:8081.

		<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns="http://predic8.com/wsdl/shop/1/">
		   <soapenv:Header/>
		   <soapenv:Body>
		      <ns:create>
		         <customerId>001</customerId>
		      </ns:create>
		   </soapenv:Body>
		</soapenv:Envelope>

	To make SOAP requests to your running app, use a service such as [SoapUI](http://www.soapui.org/), a free software that saves you a few steps by automatically providing the SOAP message structure you need for each kind of request that can be made to the API.
	
	In SoapUI:
	
    1. Go to File > Preferences and uncheck the Response compression option (otherwise the API will return a Gzip payload, which needs some extra steps to be handled)
	1.     Create a new project
	1.     Provide it with the WSDL URI: http://www.predic8.com:8080/shop/ShopService?wsdl
	1.     Direct the requests to your localhost address: http://localhost:8081
	
	You can instead use a browser extension such as [Postman](https://chrome.google.com/webstore/detail/postman-rest-client/fdmmgilgnpjigdojojpjoooidkmcomcm) (Google Chrome), or the curl command line utility. Remember that to use these, you must know the required structure of the requests. 
	
	As a response to this request, you should receive a SOAP envelope with a field named cartId, indicating that the user you created now owns the shopping cart under this ID.

### How it Works ###

Follow the anatomy described here to build a proxy application in Mule Studio that abstracts your API to a new layer. Your proxy application needs to:

1. Accept incoming service calls from applications and route them to the URI of your target API.
1. Verify that the request's structure matches what is specified in the WSDL file.
1. Copy any message headers from the service call into a format that can be passed along to your API, without passing on the headers that are generated internally by Mule.
1. Capture message headers from your API's response and attach them to the response message, without passing on the headers that are generated by Mule.
1. Once your API has issued a response, remove the message header named connection
1. Route the response back to the application that made the service call.

The following steps outline the process to build this application.

1. Create a new Mule Project by going to **File > New... > Mule Project**, naming it **SOAPproxy**.
1. Drag an **HTTP endpoint** into a new flow. This is an inbound endpoint for your proxy application and receives all the requests that reference its address.

	If you click on the HTTP icon, Studio opens the endpoint's properties editor in the console below the canvas. Here, you can configure the inbound endpoint to be reached through a custom address by setting the host and port, or switching to the advanced tab and specifying an address. For the purposes of this example, keep all the default settings in the General tab, except for the Display Name, which you can change to Receive HTTP Requests From Apps.

		Display Name	Receive HTTP Requests From Apps
		Host			localhost
		Port			8081
		Path	 
1. Drag a **second HTTP** endpoint into the same flow. This second endpoint acts as an outbound endpoint and passes requests to your API, it receives responses from the API, and passes the responses back to the application that initiated the API call. 
2. Click on the second HTTP icon to open its properties editor. Configure the General tab: 
	
		Display Name	Send Requests to API
		Host			www.predic8.com
		Port			8080
		Path			shop/ShopService
		Method			POST

1. The host, port, and path point to the API that this proxy accesses. For SOAP APIs, the method should always be set to POST.
1. Drag a **SOAP component** in between your two HTTP endpoints. This component checks the API's WSDL description file and verifies that the incoming request adheres to the required structure. It also modifies the payload from a SOAP envelope to just its body.
2. Click the SOAP icon to open its properties editor, and configure it as shown.

		Display Name	Check WSDL
		Operation		Proxy service
		Port			ShopServicePTPort
		Namespace		http://predic8.com/wsdl/shop/1/
		Service			ShopService
		Payload			envelope

	Note that the Namespace is case-sensitive.

2. Click on the **Advanced** tab to open the advanced options of the SOAP component and set the **WSDL Location** to http://www.predic8.com:8080/shop/ShopService?wsdl.
	This API happens to use SOAP 1.1, which is the default. If your API uses SOAP 1.2, be sure to select this for your proxy application.
8. Add a second **SOAP component** to process the output of the previous one. This new component prepares the incoming message to be sent through the HTTP outbound endpoint.
9. Click the SOAP icon to open its properties editor, and configure it as shown.

		Display Name	Proxy Client
		Operation		Proxy client
		Payload			envelope

	This SOAP component uses the proxy-client operation to prepare the request for the SOAP API.
10. To deal with the message headers, you'll need to perform a series of simple operations. As these same operations have to be processed both with incoming and outgoing messages, it makes sense to encapsulate this set of tasks into a reusable sub-flow that you can call twice. You must add:

    1. **sub-flow** outside the current flow, created by dragging a sub-flow component to the empty space below the flow 
	1. **flow reference** element after the Check WSDL SOAP component
	1. another **flow reference** element after the outbound HTTP request to your API.
	
	Whenever the message reaches one of the **flow reference **elements, the logic in the referenced **sub-flow** is executed, then the execution of the rest of the main flow is resumed.
11. Click on the sub-flow and rename it to **copy-headers**.
12. Configure both flow reference elements so that they reference the sub-flow you created:
	
		Display Name	Copy HTTP Headers
		Flow name		copy-headers
13. When an application makes a call to your API, that call may include headers that the API needs to receive. The proxy application must capture all incoming headers and pass them along, unaltered. In Mule, any incoming message headers that enter the proxy application are treated as inbound properties, which are not forwarded to your API. Thus, the proxy application must take HTTP inbound properties and transform them into **outbound properties**, which are sent to the API via the outbound endpoint. In the new sub-flow that you created below the main flow, drag and drop an **expression** component.
14. Configure this component as shown below.

		Display Name 	Copy all HTTP Headers
		Expression		message.outboundProperties.putAll(message.inboundProperties['http.headers']);
15. Still in the sub-flow, after the expression component, add these **property** transformers and configure them as follows:
	
	First Property transformer:
		
		Display Name	Remove Content Length
		Operation		Remove Property
		Name			Content-Length

	Second Property transformer:

		Display Name	Remove MULE properties
		Operation		Remove Property
		Name			MULE_*

	Third Property transformer:

		Display Name	Remove X_MULE properties
		Operation		Remove Property
		Name			X_MULE*	

	The output of a well-built proxy should be identical to its input. Mule auto-generates a few headers that are meant for using within the flow and that are irrelevant to your API, when making all HTTP headers into outbound properties (as instructed in the previous step) these headers will be passed on as well. The Property transformers covered in this step take care of removing these unnecessary headers.

16. After receiving a response from the API, the connection header will mark the communication as complete. This causes an error, as you still want this response from your API to be routed back to the app that sent the request.

This issue is solved by removing the connection property. Add another **property** transformer in your main flow, after the second reference to the Copy HTTP Headers sub-flow.
17. Configure the transformer as follows:

		Display Name	Remove Connection Header
		Operation		Remove Property
		Name			Connection
You now have a minimum proxy that can route requests and responses to and from your SOAP API.

### Documentation ###

Studio includes a feature that enables you to easily export all the documentation you have recorded for your project. Whenever you want to share your project with others outside the Studio environment, you can export the project's documentation to print, email or share online. Studio's auto-generated documentation includes:

- A visual diagram of the flows in your application
- The XML configuration which corresponds to each flow in your application
- The text you entered in the Notes tab of any building block in your flow

Follow [the procedure](http://www.mulesoft.org/documentation/display/current/Importing+and+Exporting+in+Studio#ImportingandExportinginStudio-ExportingStudioDocumentation) to export auto-generated Studio documentation.

### Go Further 

- Mulesoft offer an out of the box solution for proxying and managing existing APIs using the Anypoint API Platform. To see the detailed documentation and capabilities, please refer to: [Proxying Your API](http://www.mulesoft.org/documentation/display/current/Proxying+Your+API)